=== modified file 'bzrlib/smart/branch.py'
--- bzrlib/smart/branch.py	2007-04-20 05:11:46 +0000
+++ bzrlib/smart/branch.py	2007-04-24 12:20:09 +0000
@@ -19,7 +19,11 @@
 
 from bzrlib import errors
 from bzrlib.bzrdir import BzrDir
-from bzrlib.smart.request import SmartServerRequest, SmartServerResponse
+from bzrlib.smart.request import (
+    FailedSmartServerResponse,
+    SmartServerRequest,
+    SuccessfulSmartServerResponse,
+    )
 
 
 class SmartServerBranchRequest(SmartServerRequest):
@@ -73,7 +77,7 @@
             content = branch.control_files.get('branch.conf').read()
         except errors.NoSuchFile:
             content = ''
-        return SmartServerResponse( ('ok', ), content)
+        return SuccessfulSmartServerResponse( ('ok', ), content)
 
 
 class SmartServerRequestRevisionHistory(SmartServerBranchRequest):
@@ -84,7 +88,7 @@
         The revision list is returned as the body content,
         with each revision utf8 encoded and \x00 joined.
         """
-        return SmartServerResponse(
+        return SuccessfulSmartServerResponse(
             ('ok', ), ('\x00'.join(branch.revision_history())))
 
 
@@ -96,7 +100,7 @@
         The revno is encoded in decimal, the revision_id is encoded as utf8.
         """
         revno, last_revision = branch.last_revision_info()
-        return SmartServerResponse(('ok', str(revno), last_revision))
+        return SuccessfulSmartServerResponse(('ok', str(revno), last_revision))
 
 
 class SmartServerBranchRequestSetLastRevision(SmartServerLockedBranchRequest):
@@ -106,10 +110,10 @@
             branch.set_revision_history([])
         else:
             if not branch.repository.has_revision(new_last_revision_id):
-                return SmartServerResponse(
+                return FailedSmartServerResponse(
                     ('NoSuchRevision', new_last_revision_id))
             branch.generate_revision_history(new_last_revision_id)
-        return SmartServerResponse(('ok',))
+        return SuccessfulSmartServerResponse(('ok',))
 
 
 class SmartServerBranchRequestLockWrite(SmartServerBranchRequest):
@@ -126,15 +130,15 @@
             finally:
                 branch.repository.unlock()
         except errors.LockContention:
-            return SmartServerResponse(('LockContention',))
+            return FailedSmartServerResponse(('LockContention',))
         except errors.TokenMismatch:
-            return SmartServerResponse(('TokenMismatch',))
+            return FailedSmartServerResponse(('TokenMismatch',))
         except errors.UnlockableTransport:
-            return SmartServerResponse(('UnlockableTransport',))
+            return FailedSmartServerResponse(('UnlockableTransport',))
         branch.repository.leave_lock_in_place()
         branch.leave_lock_in_place()
         branch.unlock()
-        return SmartServerResponse(('ok', branch_token, repo_token))
+        return SuccessfulSmartServerResponse(('ok', branch_token, repo_token))
 
 
 class SmartServerBranchRequestUnlock(SmartServerBranchRequest):
@@ -147,9 +151,9 @@
             finally:
                 branch.repository.unlock()
         except errors.TokenMismatch:
-            return SmartServerResponse(('TokenMismatch',))
+            return FailedSmartServerResponse(('TokenMismatch',))
         branch.repository.dont_leave_lock_in_place()
         branch.dont_leave_lock_in_place()
         branch.unlock()
-        return SmartServerResponse(('ok',))
+        return SuccessfulSmartServerResponse(('ok',))
         

=== modified file 'bzrlib/smart/bzrdir.py'
--- bzrlib/smart/bzrdir.py	2007-04-18 05:10:19 +0000
+++ bzrlib/smart/bzrdir.py	2007-04-24 12:20:09 +0000
@@ -19,7 +19,11 @@
 
 from bzrlib import errors
 from bzrlib.bzrdir import BzrDir, BzrDirFormat
-from bzrlib.smart.request import SmartServerRequest, SmartServerResponse
+from bzrlib.smart.request import (
+    FailedSmartServerResponse,
+    SmartServerRequest,
+    SuccessfulSmartServerResponse,
+    )
 
 
 class SmartServerRequestOpenBzrDir(SmartServerRequest):
@@ -35,7 +39,7 @@
             answer = 'no'
         else:
             answer = 'yes'
-        return SmartServerResponse((answer,))
+        return SuccessfulSmartServerResponse((answer,))
 
 
 class SmartServerRequestFindRepository(SmartServerRequest):
@@ -68,9 +72,9 @@
                 tree_ref = 'yes'
             else:
                 tree_ref = 'no'
-            return SmartServerResponse(('ok', '/'.join(segments), rich_root, tree_ref))
+            return SuccessfulSmartServerResponse(('ok', '/'.join(segments), rich_root, tree_ref))
         except errors.NoRepositoryPresent:
-            return SmartServerResponse(('norepository', ))
+            return FailedSmartServerResponse(('norepository', ))
 
 
 class SmartServerRequestInitializeBzrDir(SmartServerRequest):
@@ -83,7 +87,7 @@
         """
         target_transport = self._backing_transport.clone(path)
         BzrDirFormat.get_default_format().initialize_on_transport(target_transport)
-        return SmartServerResponse(('ok', ))
+        return SuccessfulSmartServerResponse(('ok', ))
 
 
 class SmartServerRequestOpenBranch(SmartServerRequest):
@@ -98,8 +102,8 @@
         try:
             reference_url = bzrdir.get_branch_reference()
             if reference_url is None:
-                return SmartServerResponse(('ok', ''))
+                return SuccessfulSmartServerResponse(('ok', ''))
             else:
-                return SmartServerResponse(('ok', reference_url))
+                return SuccessfulSmartServerResponse(('ok', reference_url))
         except errors.NotBranchError:
-            return SmartServerResponse(('nobranch', ))
+            return FailedSmartServerResponse(('nobranch', ))

=== modified file 'bzrlib/smart/protocol.py'
--- bzrlib/smart/protocol.py	2007-04-24 12:07:30 +0000
+++ bzrlib/smart/protocol.py	2007-04-24 12:20:09 +0000
@@ -133,6 +133,7 @@
     def _send_response(self, response):
         """Send a smart server response down the output stream."""
         assert not self._finished, 'response already sent'
+        response.is_successful()
         args = response.args
         body = response.body
         self._finished = True

=== modified file 'bzrlib/smart/repository.py'
--- bzrlib/smart/repository.py	2007-04-16 17:23:40 +0000
+++ bzrlib/smart/repository.py	2007-04-24 12:20:09 +0000
@@ -19,7 +19,11 @@
 
 from bzrlib import errors
 from bzrlib.bzrdir import BzrDir
-from bzrlib.smart.request import SmartServerRequest, SmartServerResponse
+from bzrlib.smart.request import (
+    FailedSmartServerResponse,
+    SmartServerRequest,
+    SuccessfulSmartServerResponse,
+    )
 
 
 class SmartServerRepositoryRequest(SmartServerRequest):
@@ -61,12 +65,12 @@
             # Note that we return an empty body, rather than omitting the body.
             # This way the client knows that it can always expect to find a body
             # in the response for this method, even in the error case.
-            return SmartServerResponse(('nosuchrevision', revision_id), '')
+            return FailedSmartServerResponse(('nosuchrevision', revision_id), '')
 
         for revision, parents in revision_graph.items():
             lines.append(' '.join([revision,] + parents))
 
-        return SmartServerResponse(('ok', ), '\n'.join(lines))
+        return SuccessfulSmartServerResponse(('ok', ), '\n'.join(lines))
 
 
 class SmartServerRequestHasRevision(SmartServerRepositoryRequest):
@@ -80,9 +84,9 @@
             present.
         """
         if repository.has_revision(revision_id):
-            return SmartServerResponse(('yes', ))
+            return SuccessfulSmartServerResponse(('yes', ))
         else:
-            return SmartServerResponse(('no', ))
+            return SuccessfulSmartServerResponse(('no', ))
 
 
 class SmartServerRepositoryGatherStats(SmartServerRepositoryRequest):
@@ -125,7 +129,7 @@
         if stats.has_key('size'):
             body += 'size: %d\n' % stats['size']
 
-        return SmartServerResponse(('ok', ), body)
+        return SuccessfulSmartServerResponse(('ok', ), body)
 
 
 class SmartServerRepositoryIsShared(SmartServerRepositoryRequest):
@@ -138,9 +142,9 @@
             shared, and ('no', ) if it is not.
         """
         if repository.is_shared():
-            return SmartServerResponse(('yes', ))
+            return SuccessfulSmartServerResponse(('yes', ))
         else:
-            return SmartServerResponse(('no', ))
+            return SuccessfulSmartServerResponse(('no', ))
 
 
 class SmartServerRepositoryLockWrite(SmartServerRepositoryRequest):
@@ -152,14 +156,14 @@
         try:
             token = repository.lock_write(token=token)
         except errors.LockContention, e:
-            return SmartServerResponse(('LockContention',))
+            return FailedSmartServerResponse(('LockContention',))
         except errors.UnlockableTransport:
-            return SmartServerResponse(('UnlockableTransport',))
+            return FailedSmartServerResponse(('UnlockableTransport',))
         repository.leave_lock_in_place()
         repository.unlock()
         if token is None:
             token = ''
-        return SmartServerResponse(('ok', token))
+        return SuccessfulSmartServerResponse(('ok', token))
 
 
 class SmartServerRepositoryUnlock(SmartServerRepositoryRequest):
@@ -168,8 +172,8 @@
         try:
             repository.lock_write(token=token)
         except errors.TokenMismatch, e:
-            return SmartServerResponse(('TokenMismatch',))
+            return FailedSmartServerResponse(('TokenMismatch',))
         repository.dont_leave_lock_in_place()
         repository.unlock()
-        return SmartServerResponse(('ok',))
+        return SuccessfulSmartServerResponse(('ok',))
 

=== modified file 'bzrlib/smart/request.py'
--- bzrlib/smart/request.py	2007-04-24 12:07:30 +0000
+++ bzrlib/smart/request.py	2007-04-24 12:20:09 +0000
@@ -77,7 +77,11 @@
 
 
 class SmartServerResponse(object):
-    """Response generated by SmartServerRequestHandler."""
+    """A response to a client request.
+    
+    This base class should not be used. Instead use
+    SuccessfulSmartServerResponse and FailedSmartServerResponse as appropriate.
+    """
 
     def __init__(self, args, body=None):
         self.args = args
@@ -89,7 +93,8 @@
         return other.args == self.args and other.body == self.body
 
     def __repr__(self):
-        return "<SmartServerResponse args=%r body=%r>" % (self.args, self.body)
+        return "<SmartServerResponse args=%r body=%r>" % (self.is_successful(), 
+            self.args, self.body)
 
 
 class FailedSmartServerResponse(SmartServerResponse):
@@ -187,13 +192,13 @@
         try:
             return callable(*args, **kwargs)
         except errors.NoSuchFile, e:
-            return SmartServerResponse(('NoSuchFile', e.path))
+            return FailedSmartServerResponse(('NoSuchFile', e.path))
         except errors.FileExists, e:
-            return SmartServerResponse(('FileExists', e.path))
+            return FailedSmartServerResponse(('FileExists', e.path))
         except errors.DirectoryNotEmpty, e:
-            return SmartServerResponse(('DirectoryNotEmpty', e.path))
+            return FailedSmartServerResponse(('DirectoryNotEmpty', e.path))
         except errors.ShortReadvError, e:
-            return SmartServerResponse(('ShortReadvError',
+            return FailedSmartServerResponse(('ShortReadvError',
                 e.path, str(e.offset), str(e.length), str(e.actual)))
         except UnicodeError, e:
             # If it is a DecodeError, than most likely we are starting
@@ -206,11 +211,11 @@
             else:
                 val = 's:' + str_or_unicode.encode('base64')
             # This handles UnicodeEncodeError or UnicodeDecodeError
-            return SmartServerResponse((e.__class__.__name__,
+            return FailedSmartServerResponse((e.__class__.__name__,
                     e.encoding, val, str(e.start), str(e.end), e.reason))
         except errors.TransportNotPossible, e:
             if e.msg == "readonly transport":
-                return SmartServerResponse(('ReadOnlyError', ))
+                return FailedSmartServerResponse(('ReadOnlyError', ))
             else:
                 raise
 
@@ -221,7 +226,7 @@
     """
 
     def do(self):
-        return SmartServerResponse(('ok', '2'))
+        return SuccessfulSmartServerResponse(('ok', '2'))
 
 
 class GetBundleRequest(SmartServerRequest):
@@ -236,7 +241,7 @@
         base_revision = revision.NULL_REVISION
         write_bundle(repo, revision_id, base_revision, tmpf)
         tmpf.seek(0)
-        return SmartServerResponse((), tmpf.read())
+        return SuccessfulSmartServerResponse((), tmpf.read())
 
 
 class SmartServerIsReadonly(SmartServerRequest):
@@ -247,7 +252,7 @@
             answer = 'yes'
         else:
             answer = 'no'
-        return SmartServerResponse((answer,))
+        return SuccessfulSmartServerResponse((answer,))
 
 
 request_handlers = registry.Registry()

=== modified file 'bzrlib/smart/vfs.py'
--- bzrlib/smart/vfs.py	2007-04-11 04:26:23 +0000
+++ bzrlib/smart/vfs.py	2007-04-24 12:20:09 +0000
@@ -64,14 +64,14 @@
 
     def do(self, relpath):
         r = self._backing_transport.has(relpath) and 'yes' or 'no'
-        return request.SmartServerResponse((r,))
+        return request.SuccessfulSmartServerResponse((r,))
 
 
 class GetRequest(VfsRequest):
 
     def do(self, relpath):
         backing_bytes = self._backing_transport.get_bytes(relpath)
-        return request.SmartServerResponse(('ok',), backing_bytes)
+        return request.SuccessfulSmartServerResponse(('ok',), backing_bytes)
 
 
 class AppendRequest(VfsRequest):
@@ -83,14 +83,14 @@
     def do_body(self, body_bytes):
         old_length = self._backing_transport.append_bytes(
             self._relpath, body_bytes, self._mode)
-        return request.SmartServerResponse(('appended', '%d' % old_length))
+        return request.SuccessfulSmartServerResponse(('appended', '%d' % old_length))
 
 
 class DeleteRequest(VfsRequest):
 
     def do(self, relpath):
         self._backing_transport.delete(relpath)
-        return request.SmartServerResponse(('ok', ))
+        return request.SuccessfulSmartServerResponse(('ok', ))
 
 
 class IterFilesRecursiveRequest(VfsRequest):
@@ -98,14 +98,14 @@
     def do(self, relpath):
         transport = self._backing_transport.clone(relpath)
         filenames = transport.iter_files_recursive()
-        return request.SmartServerResponse(('names',) + tuple(filenames))
+        return request.SuccessfulSmartServerResponse(('names',) + tuple(filenames))
 
 
 class ListDirRequest(VfsRequest):
 
     def do(self, relpath):
         filenames = self._backing_transport.list_dir(relpath)
-        return request.SmartServerResponse(('names',) + tuple(filenames))
+        return request.SuccessfulSmartServerResponse(('names',) + tuple(filenames))
 
 
 class MkdirRequest(VfsRequest):
@@ -113,14 +113,14 @@
     def do(self, relpath, mode):
         self._backing_transport.mkdir(relpath,
                                       _deserialise_optional_mode(mode))
-        return request.SmartServerResponse(('ok',))
+        return request.SuccessfulSmartServerResponse(('ok',))
 
 
 class MoveRequest(VfsRequest):
 
     def do(self, rel_from, rel_to):
         self._backing_transport.move(rel_from, rel_to)
-        return request.SmartServerResponse(('ok',))
+        return request.SuccessfulSmartServerResponse(('ok',))
 
 
 class PutRequest(VfsRequest):
@@ -131,7 +131,7 @@
 
     def do_body(self, body_bytes):
         self._backing_transport.put_bytes(self._relpath, body_bytes, self._mode)
-        return request.SmartServerResponse(('ok',))
+        return request.SuccessfulSmartServerResponse(('ok',))
 
 
 class PutNonAtomicRequest(VfsRequest):
@@ -149,7 +149,7 @@
                 mode=self._mode,
                 create_parent_dir=self._create_parent,
                 dir_mode=self._dir_mode)
-        return request.SmartServerResponse(('ok',))
+        return request.SuccessfulSmartServerResponse(('ok',))
 
 
 class ReadvRequest(VfsRequest):
@@ -162,7 +162,7 @@
         offsets = self._deserialise_offsets(body_bytes)
         backing_bytes = ''.join(bytes for offset, bytes in
             self._backing_transport.readv(self._relpath, offsets))
-        return request.SmartServerResponse(('readv',), backing_bytes)
+        return request.SuccessfulSmartServerResponse(('readv',), backing_bytes)
 
     def _deserialise_offsets(self, text):
         # XXX: FIXME this should be on the protocol object.
@@ -179,20 +179,20 @@
 
     def do(self, rel_from, rel_to):
         self._backing_transport.rename(rel_from, rel_to)
-        return request.SmartServerResponse(('ok', ))
+        return request.SuccessfulSmartServerResponse(('ok', ))
 
 
 class RmdirRequest(VfsRequest):
 
     def do(self, relpath):
         self._backing_transport.rmdir(relpath)
-        return request.SmartServerResponse(('ok', ))
+        return request.SuccessfulSmartServerResponse(('ok', ))
 
 
 class StatRequest(VfsRequest):
 
     def do(self, relpath):
         stat = self._backing_transport.stat(relpath)
-        return request.SmartServerResponse(
+        return request.SuccessfulSmartServerResponse(
             ('stat', str(stat.st_size), oct(stat.st_mode)))
 

=== modified file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	2007-04-24 12:07:30 +0000
+++ bzrlib/tests/test_smart_transport.py	2007-04-24 12:20:09 +0000
@@ -1267,7 +1267,7 @@
             def do_body(cmd, body_bytes):
                 self.end_received = True
                 self.assertEqual('abcdefg', body_bytes)
-                return request.SmartServerResponse(('ok', ))
+                return request.SuccessfulSmartServerResponse(('ok', ))
         smart_protocol.request._command = FakeCommand()
         # Call accept_bytes to make sure that internal state like _body_decoder
         # is initialised.  This test should probably be given a clearer
@@ -1409,6 +1409,13 @@
             request.SuccessfulSmartServerResponse(('x',)))
         self.assertEqual(0, smart_protocol.next_read_size())
 
+    def test__send_response_errors_with_base_response(self):
+        """Ensure that only the Successful/Failed subclasses are used."""
+        smart_protocol = protocol.SmartServerRequestProtocolOne(
+            None, lambda x: None)
+        self.assertRaises(AttributeError, smart_protocol._send_response,
+            request.SmartServerResponse(('x',)))
+
     def test_query_version(self):
         """query_version on a SmartClientProtocolOne should return a number.
         
@@ -1627,6 +1634,13 @@
             request.SuccessfulSmartServerResponse(('x',)))
         self.assertEqual(0, smart_protocol.next_read_size())
 
+    def test__send_response_errors_with_base_response(self):
+        """Ensure that only the Successful/Failed subclasses are used."""
+        smart_protocol = protocol.SmartServerRequestProtocolOne(
+            None, lambda x: None)
+        self.assertRaises(AttributeError, smart_protocol._send_response,
+            request.SmartServerResponse(('x',)))
+
     def test_query_version(self):
         """query_version on a SmartClientProtocolTwo should return a number.
         

